#!/usr/bin/perl

use strict;
use warnings;

# Setup Environment
BEGIN
{
  die "GSDLHOME not set\n" unless defined $ENV{'GSDLHOME'};
  die "GSDLOS not set\n" unless defined $ENV{'GSDLOS'};
  die "GEXTPARALLELBUILDING_INSTALLED not set\n" unless defined $ENV{'GEXTPARALLELBUILDING_INSTALLED'};
  die "PERL_VERSION not set\n" unless defined $ENV{'PERL_VERSION'};
  # Bit::Vector and Thrift modules
  unshift (@INC, $ENV{'GEXTPARALLELBUILDING'} . '/' . $ENV{'GSDLOS'} . '/lib/perl/' . $ENV{'PERL_VERSION'});
  # ThriftFS Perl API
  unshift (@INC, $ENV{'GEXTPARALLELBUILDING'} . '/packages/hadoop-1.1.0/contrib/thriftfs/gen-perl');
}

use Thrift::Socket;
use Thrift::BufferedTransport;
use Thrift::BinaryProtocol;
use HadoopFS::FileSystem;
use Getopt::Long;
use POSIX qw(strftime);

my $host = $ARGV[0] || "localhost";
my $port = $ARGV[1] || 58660;
my $pathname = $ARGV[2] || "/";

print "Create Thrift Client to $host:$port\n";
my $socket = Thrift::Socket->new ($host, $port);
$socket->setSendTimeout (10000);
$socket->setRecvTimeout (20000);

my $transport = Thrift::BufferedTransport->new ($socket);
my $protocol = Thrift::BinaryProtocol->new ($transport);
my $client = HadoopFS::FileSystemClient->new ($protocol);

eval {
 $transport->open ();
};
if ($@)
{
 print "Unable to connect: $@->{message}\n";
 exit 1;
}

# Create a file and write some data to it
my $filename = "/foo.txt";
print "File: $filename\n";
my $path = HadoopFS::Pathname->new ( { pathname => $filename } );
if ($client->exists ($path))
{
 # Delete the file if it exists
 print "  ... file exists.\n";
 $client->rm ($path);
 print "  ... deleted.\n";
}

print "  ... creating\n";
my $fh = $client->create ( $path );
print "  ... writing data\n";
$client->write ($fh, "Hello, World!\n");
$client->close ($fh);
print "  ... closed\n";

# Read the contents of the file
eval {
 print "  ... re-opening\n";
 $fh = $client->open ($path);
 my $data = undef;
 my $offset = 0;
 my $length = 4096;
 print "  ... reading data\n";
 while ($data = $client->read ($fh, $offset, $length)) # Read 4K blocks at a time
 {
 print "$filename: $data";
 last if (length ($data) < $length);
 $offset += $length;
 print "\n";
 }
 $client->close ($fh);
 print "  ... closed\n";
};
if ($@)
{
 print $@->{message} . "\n";
}

print "\n";
# Get information about the file we just created
my $file = $client->stat ($path);
print "File stat:\n";
printf(" ... Filename: %s\n", $file->{path});
printf(" ... Length: %s\n", $file->{length});
printf(" ... Block Replication: %s\n", ($file->{blockReplication} ? $file->{blockReplication} : "-"));
printf(" ... Block Size: %s\n", ($file->{blockSize} ? $file->{blockSize} : "-"));
printf(" ... Modification Time: %s\n", strftime ("%Y-%m-%d %H:%M:%S", localtime ($file->{modificationTime} / 1000)));
printf(" ... Directory: %s\n", ($file->{isdir} ? "yes" : "no"));
printf(" ... Permission: %s\n", $file->{permission});
printf(" ... Owner: %s\n", $file->{owner});
printf(" ... Group: %s\n", $file->{group});

print "\n";
# Get the file blocks of this file
print "File Blocks:\n";
my $blocks = $client->getFileBlockLocations ($path, 0, 4096);
foreach my $block (@{$blocks})
{
 printf ("... Offset: %d\n", $block->{offset});
 printf ("... Length: %d\n", $block->{length});
 printf ("... Hosts:\n");
 foreach my $datanode (@{$block->{names}})
 {
 print "      $datanode\n";
 }
}

# Using ls
print "\n";
print "ls /:\n";
$path = HadoopFS::Pathname->new ( { pathname => $pathname } );
my $files = $client->listStatus ( $path );
if ($files && @{$files} > 0)
{
 printf ("Found %d items\n", scalar @{$files});
}
foreach my $file (@{$files})
{
 my $pathname = $file->{path};
 my $hdfshost;
 if ($pathname =~ m/^hdfs:\/\/([^\/])+(\/.+)$/)
 {
 $hdfshost = $1;
 $pathname = $2;
 }
 printf("%s%s %-3s %-9s %-9s %10s %s %s\n",
 $file->{isdir} ? "d" : "-",
 $file->{permission},
 $file->{blockReplication} ? $file->{blockReplication} : "-",
 $file->{owner},
 $file->{group},
 $file->{length},
 strftime ("%Y-%m-%d %H:%M", localtime ($file->{modificationTime})),
 $pathname
 );
}

$transport->close ();

exit 0;
